#!/usr/bin/env python

"""
HTML-to-Moin translator.

Copyright (C) 2023 Paul Boddie <paul@boddie.org.uk>

This program is free software; you can redistribute it and/or modify it under
the terms of the GNU General Public License as published by the Free Software
Foundation; either version 3 of the License, or (at your option) any later
version.

This program is distributed in the hope that it will be useful, but WITHOUT
ANY WARRANTY; without even the implied warranty of MERCHANTABILITY or FITNESS
FOR A PARTICULAR PURPOSE.  See the GNU General Public License for more
details.

You should have received a copy of the GNU General Public License along with
this program.  If not, see <http://www.gnu.org/licenses/>.
"""

from moinformat.translators.common import Translator
from moinformat.tree.moin import Block, Heading, Region, Text

def int_or_default(s, default):
    if not s:
        return default
    try:
        return int(s)
    except ValueError:
        return default

class HTMLToMoinTranslator(Translator):

    "Translation of HTML document nodes to Moin document nodes."

    input_formats = ["html"]
    formats = ["moin"]

    def _get_attribute(self, element, name):
        for attribute in element.attributes:
            if attribute.name == name:
                return attribute.value and attribute.value.value
        return None

    def _get_class_values(self, element):
        class_value = self._get_attribute(element, "class")
        if not class_value:
            return {}

        d = {}
        for token in class_value.split():
            if token and token.startswith("region-"):
                _region, name, value = token.split("-", 2)
                d[name] = value
        return d

    def element(self, element):
        if not element.name:
            return None
        elif element.name[0] == "h" and element.name[1:].isdigit():
            return Heading(self.container(element), int(element.name[1:]),
                           start_pad=" ", end_pad=" ", end_extra="\n",
                           identifier=self._get_attribute(element, "id"))
        elif element.name == "p":
            return Block(self.container(element))
        elif element.name == "span":
            d = self._get_class_values(element)
            if d.has_key("type"):
                return Region(self.container(element),
                              int_or_default(d.get("level"), 0),
                              int_or_default(d.get("indent"), 0),
                              d.get("type"),
                              extra="\n")
            else:
                return Block(self.container(element))
        else:
            return None

    def fragment(self, fragment):
        return self.container(fragment)

    def text(self, text):
        return Text(text.value)

    # Some nodes are not directly translated.

    def node(self, node):
        return None

    attribute = node
    attribute_value = node
    comment = node
    directive = node
    inclusion = node

translator = HTMLToMoinTranslator

# vim: tabstop=4 expandtab shiftwidth=4
